import request from "request-promise-native";
import { Response } from "request";
import { IAggregator, INewsItem } from "./aggregator";
import { getRepository } from "typeorm";
import { AggroNewsItem } from "@shared/db/entity/AggroNewsItem";
import cheerio from "cheerio";

const updatePage = "http://com3d2.jp/update/";
const changeLogPattern = /\[\s*([^\s\]]+)\s*\]\s*((・.*)\s+)+/gim;
const FEED_NAME = "com3d2-jp-updates";

function getVersionNumber(verStr: string) {
    let verPart = verStr.replace(/[\.\s]/g, "");
    if(verPart.length < 4)
        verPart += "0";
    return +verPart;
}

async function aggregate() {
    let repo = getRepository(AggroNewsItem);
    
    let lastPost = await repo.findOne({
        select: [ "newsId" ],
        where: { feedName: FEED_NAME },
        order: { newsId: "DESC" }
    });

    if(!lastPost)
        lastPost = repo.create({
            newsId: 0
        });
    
    try {
        let mainPageRes = await request(updatePage, {resolveWithFullResponse: true}) as Response;

        if(mainPageRes.statusCode != 200)
            return;
        
        let rootNode = cheerio.load(mainPageRes.body);

        let readme = rootNode("div.readme");

        if(!readme) {
            console.log("[COM3D2 JP UPDATE] Failed to find listing!");
            return [];
        }

        let latestVersionChangelog = changeLogPattern.exec(readme.text());

        if(!latestVersionChangelog)
            return [];

        let version = getVersionNumber(latestVersionChangelog[1]);
        let text = latestVersionChangelog[0];

        if(version <= lastPost.newsId)
            return [];
        
        return [{
            feedId: FEED_NAME,
            newsId: version,
            link: updatePage,
            title: latestVersionChangelog[1],
            author: "COM3D2 UPDATE",
            contents: text,
            embedColor: 0xcccccc,
            needsTranslation: true
        }] as INewsItem[];
    } catch(err) {
        return [];
    }
}

export default {
    aggregate: aggregate
} as IAggregator;